;;; Lisplab, level1-classes.lisp
;;; Level1, abstract matrix classes

;;; Copyright (C) 2009 Joern Inge Vestgaarden
;;;
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2 of the License, or
;;; (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License along
;;; with this program; if not, write to the Free Software Foundation, Inc.,
;;; 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

;;; The class structure is inspired by the stream example
;;; in Object-Oriented programming in Common Lisp, by Sonja E. Keene. 


(in-package :lisplab) 

;;; The matrix element tells the element type of the matrix

(defclass matrix-element-base () 
  ((element-type
    :allocation :class
    :initform t
    :reader element-type)))

(defclass matrix-element-number (matrix-element-base) 
  ((element-type
    :allocation :class
    :initform 'number
    :reader element-type)))

(defclass matrix-element-complex (matrix-element-number) 
  ((element-type
    :allocation :class
    :initform 'complex
    :reader element-type)))

(defclass matrix-element-complex-double-float (matrix-element-complex) 
  ((element-type
    :allocation :class
    :initform '(complex double-float)
    :reader element-type)))

(defclass matrix-element-real (matrix-element-number) 
  ((element-type
    :allocation :class
    :initform 'real
    :reader element-type)))

(defclass matrix-element-double-float (matrix-element-real) 
  ((element-type
    :allocation :class
    :initform 'double-float
    :reader element-type)))

;;; A way to solve conflicts if there is one foreign and one native implementation

(defclass matrix-implementation-base () ())

(defclass matrix-implementation-lisp (matrix-implementation-base) ())

(defclass matrix-implementation-blas (matrix-implementation-lisp) ())

;;; The matrix structure tells the structure of the matrix

(defclass matrix-base () ())

(defclass matrix-structure-base (matrix-base) ())

(defclass matrix-structure-general (matrix-structure-base) 
  ((rows
    :initarg :rows
    :initform 0
    :reader rows
    :type type-blas-idx)
   (cols
    :initarg :cols
    :initform 0
    :reader cols
    :type type-blas-idx)
   (size
    :reader size
    :type  type-blas-idx)))

(defclass matrix-structure-square (matrix-structure-base) 
  ((rowcols
    :initarg :rowcols
    :initform 0
    :reader rowcols
    :type type-blas-idx)))

(defclass matrix-structure-diagonal (matrix-structure-square) ())

(defclass matrix-structure-bidiagonal (matrix-structure-square) ())

(defclass matrix-structure-tridiagonal (matrix-structure-square) ())