;;; Lisplab, level1-constructors.lisp
;;; A symbolic naming scheme for matrix construction.

;;; Copyright (C) 2009 Joern Inge Vestgaarden
;;;
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2 of the License, or
;;; (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License along
;;; with this program; if not, write to the Free Software Foundation, Inc.,
;;; 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

;;; TODO: maybe I should store constructors rather than symbols. 
;;;;      Then it would be possible to use this also for arrays. 

(in-package :lisplab)

;;;; Matrix constructors

(defmethod make-matrix-instance ((type standard-class) dim value)
  (make-instance type :dim dim :value value))

(defmethod make-matrix-instance ((type symbol) dim value)
  (make-matrix-instance (find-class type) dim value))

(defmethod make-matrix-instance ((description list) dim value)
  (make-matrix-instance (find-matrix-class description) dim value))

;; A scheme for matrix creations

(defvar *matrix-class-to-description* (make-hash-table))
(defvar *matrix-description-to-class* (make-hash-table :test #'equal))

(defun add-matrix-class (class element-type structure implementation)
  (setf (gethash (list element-type structure implementation)
		 *matrix-description-to-class*)
	class
	(gethash class 
		 *matrix-class-to-description* )
	(list element-type structure implementation)))

(defun find-matrix-class (description)
  (let* ((entry (gethash description
		*matrix-description-to-class*)))	 
    (unless entry
      (error "No matrix of structure ~a." description))
    entry))

(defun find-matrix-description (class)
   (let* ((entry (gethash class
			  *matrix-class-to-description*)))
    (unless entry
      (error "No matrix description of class ~a." class))
    entry))

(defun create-matrix-description (obj &key et s i)
  "A simple language to modify matrix descriptions. Uses 
the obejct as foundation of the description, but you can
override the description with the keywords."
  (let ((d0 (find-matrix-description (class-name (class-of obj)))))
    (list
     (if et et (first d0))
     (if s s (second d0))
     (if i i (third d0)))))

;;; The implementaiton of the level0 generic functions

(defmethod type-spec ((obj matrix-base))
  (find-matrix-description (class-name (class-of obj))))

(defmethod function-output-type-spec (fun (input-spec list))
  ;;; A list signifies a matrix, Should be a another type of obejct at some point!
  (cons (function-output-type-spec fun (car input-spec))
	(cdr input-spec)))

(defmethod operator-output-type-spec (fun (a list) b)
  ;;; A list signifies a matrix, Should be a another type of obejct at some point!
  (cons (operator-output-type-spec fun (car a) b)
	(cdr a)))

(defmethod operator-output-type-spec (fun a (b list))
  ;;; A list signifies a matrix, Should be a another type of obejct at some point!
  (cons (operator-output-type-spec fun a (car b))
	(cdr b)))

(defmethod operator-output-type-spec (fun (a list) (b list))
  ;;; A list signifies a matrix, Should be a another type of obejct at some point!
  ;; TODO: some better way to choose 
  (cons (operator-output-type-spec fun (car a) (car b))
	(cdr a)))