;;; Lisplab, level1-dge.lisp
;;; Diagonal double-float matrices

;;; Copyright (C) 2009 Joern Inge Vestgaarden
;;;
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2 of the License, or
;;; (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License along
;;; with this program; if not, write to the Free Software Foundation, Inc.,
;;; 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

;;; TODO: bidiagnonal matrices

;;; Note: not optimzied, but I see no good reason to optimzized them either.

(in-package :lisplab)

(defclass matrix-base-ddi 
    (matrix-structure-diagonal
     matrix-element-double-float 
     matrix-implementation-base) 
  ((diagonal-store
    :initarg :diagonal-store
    :initform nil
    :type type-blas-store)))

(defmethod initialize-instance :after ((m matrix-base-ddi) &key dim (value 0))
  (with-slots (rowcols diagonal-store) m
    (setf rowcols (if (integerp dim)
		      dim
		       (if (= (car dim) (cadr dim))
			  (car dim)
			  (error "You try to create a non-square diagonal matrix."))))
    (unless diagonal-store
      (setf diagonal-store (allocate-real-store rowcols value)))))

(defclass matrix-lisp-ddi (matrix-implementation-lisp matrix-base-ddi) ())

(defclass matrix-blas-ddi (matrix-implementation-blas matrix-lisp-ddi) ())

(defclass matrix-ddi (matrix-blas-ddi) ())

;;; Add classes to the generic matrix creation scheme
(add-matrix-class 'matrix-base-ddi :d :di :base)
(add-matrix-class 'matrix-lisp-ddi :d :di :lisp)
(add-matrix-class 'matrix-blas-ddi :d :di :blas)
(add-matrix-class 'matrix-ddi :d :di :any)

;;; Methods spezilied for the diagnoal matrices

(defmethod mref ((matrix matrix-base-ddi) row col)
  (if (= row col)
      (aref (slot-value matrix 'diagonal-store) row)
      0d0))

(defmethod (setf mref) (value (matrix  matrix-base-ddi) row col)
  (if (= row col)
      (setf (aref (slot-value matrix 'diagonal-store) row)
	    (coerce value 'double-float))
      (warn "Array out of bonds for diagonal matrix. Ignored."))) 

(defmethod size ((matrix matrix-base-ddi))
  (slot-value matrix 'rowcols))

(defmethod vref ((matrix  matrix-base-ddi) idx)
  (aref (slot-value matrix 'diagonal-store) idx))

(defmethod (setf vref) (value (matrix matrix-base-ddi) idx)
  (let ((val2 (coerce value 'double-float)))
    (setf (aref (slot-value matrix 'diagonal-store) idx)
	  val2)
    val2))
