;;; Lisplab, level1-dge.lisp
;;; General, double-float matrices

;;; Copyright (C) 2009 Joern Inge Vestgaarden
;;;
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2 of the License, or
;;; (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License along
;;; with this program; if not, write to the Free Software Foundation, Inc.,
;;; 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

(in-package :lisplab)

;;; Double float general classes

(defclass matrix-base-dge 
    (matrix-structure-general 
     matrix-element-double-float 
     matrix-implementation-base) 
  ((matrix-store
    :initarg :store
    :initform nil
    :reader matrix-store
    :type type-blas-store)))

(defmethod initialize-instance :after ((m matrix-base-dge) &key dim (value 0))
  (with-slots (rows cols size matrix-store) m
    (setf rows (car dim)
	  cols (cadr dim)
	  size (* rows cols))
    (unless matrix-store
      (setf matrix-store (allocate-real-store size value)))))

(defclass matrix-lisp-dge (matrix-implementation-lisp matrix-base-dge) ()
  (:documentation "A full matrix (rows x cols) with double float elements. 
Executes in lisp only."))

(defclass matrix-blas-dge (matrix-implementation-blas matrix-lisp-dge) ()
  (:documentation "A full matrix (rows x cols) with double float elements. 
Executes in alien blas/lapack only."))

(defclass matrix-dge (matrix-blas-dge) ()
  (:documentation "A full matrix (rows x cols) with double float matrix elements.
Executes first in alien blas/lapack if possible. If not it executes in lisp."))

;;; Add classes to the generic matrix creation scheme
(add-matrix-class 'matrix-base-dge :d :ge :base)
(add-matrix-class 'matrix-lisp-dge :d :ge :lisp)
(add-matrix-class 'matrix-blas-dge :d :ge :blas)
(add-matrix-class 'matrix-dge :d :ge :any)

;;; All leve1 methods spcialized for dge

(defmethod mref ((matrix matrix-base-dge) row col)
  (ref-blas-real-store (slot-value matrix 'matrix-store) row col (slot-value matrix 'rows)))

(defmethod (setf mref) (value (matrix  matrix-base-dge) row col)
  (let ((val2 (coerce value 'double-float)))
    (declare (type double-float val2))
    (setf (ref-blas-real-store (slot-value matrix 'matrix-store) 
			       row col (slot-value matrix 'rows))
	  val2)
    val2))

(defmethod vref ((matrix  matrix-base-dge) idx)
  (aref (the type-blas-store (slot-value matrix 'matrix-store)) idx))

(defmethod (setf vref) (value (matrix matrix-base-dge) idx)
  (let ((val2 (coerce value 'double-float)))
    (declare (type double-float val2))
    (setf (aref  (the type-blas-store (slot-value matrix 'matrix-store)) idx)
	  val2)
    val2))


