;;; Lisplab, level1-dge.lisp
;;; General, untyped matrices 

;;; Copyright (C) 2009 Joern Inge Vestgaarden
;;;
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2 of the License, or
;;; (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License along
;;; with this program; if not, write to the Free Software Foundation, Inc.,
;;; 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

(in-package :lisplab)

;;;; General matrices with unspecified element types

(defclass matrix-ge
    (matrix-structure-general matrix-element-base matrix-implementation-lisp) 
  ((matrix-store
    :initarg :store
    :initform nil
    :reader matrix-store
    :type (simple-array t (*))))
  (:documentation "A full matrix (rows x cols) with unspecified matrix element types.")) 

;;; Add class to the generic matrix creation scheme
(add-matrix-class 'matrix-base-ge :any :ge :base)
(add-matrix-class 'matrix-ge :any :ge :any)

(defmethod initialize-instance :after ((m matrix-ge) &key dim (value 0))
  (with-slots (rows cols size matrix-store) m
    (setf rows (car dim)
	  cols (cadr dim)
	  size (* rows cols))
    (unless matrix-store
      (setf matrix-store (make-array size :initial-element value)))))

;;; Level methods specialized for untyped, general matrices

(defmethod mref ((matrix matrix-ge) row col)
  (aref (slot-value matrix 'matrix-store) 
	(column-major-idx row col (slot-value matrix 'rows))))

(defmethod (setf mref) (value (matrix  matrix-ge) row col)
  (setf (aref (slot-value matrix 'matrix-store) 
	      (column-major-idx row col (slot-value matrix 'rows)))
	value))

(defmethod vref ((matrix  matrix-ge) idx)
  (aref (slot-value matrix 'matrix-store) idx))

(defmethod (setf vref) (value (matrix matrix-ge) idx)
  (setf (aref (slot-value matrix 'matrix-store) idx)
	value))
