;;; Lisplab, level1-interface.lisp
;;; Minimum matrix interface. Anything that specializes on these 
;;; generic functions is a matrix!

;;; Copyright (C) 2009 Joern Inge Vestgaarden
;;;
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2 of the License, or
;;; (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License along
;;; with this program; if not, write to the Free Software Foundation, Inc.,
;;; 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

(in-package :lisplab)

(defvar *lisplab-print-size* 10 "Suggested number of rows and columns printed to standard output. Not all matrices, such as ordinary lisp arrays, will care about the value.")

(defgeneric make-matrix-instance (type dim value)
  (:documentation "Creates a new matrix instance"))

(defgeneric ref (matrix &rest subscripts)
  (:documentation "A general accessor."))

(defgeneric (setf ref) (value matrix &rest subscripts))

(defgeneric mref (matrix row col)
  (:documentation "Matrix accessor."))

(defgeneric (setf mref) (value matrix row col))

(defgeneric vref (matrix idx)
  (:documentation "Vector accessor."))

(defgeneric (setf vref) (value matrix idx))

(defgeneric dim (matrix &optional direction)
  (:documentation "Gives a list of all dimension lengths, or the length of 
the dimension specified."))

(defgeneric (setf dim) (value matrix &optional direction))

(defgeneric element-type (matrix)
  (:documentation "The object element type, or t if any."))

(defgeneric (setf element-type) (value matrix))

(defgeneric size (matrix)
  (:documentation "Gives the number of elements in the object."))

(defgeneric (setf size) (value matrix))

(defgeneric rank (matrix)
  (:documentation "The rank is the number of dimensions."))

(defgeneric (setf rank) (value matrix))

(defgeneric rows (matrix)
  (:documentation "The number of rows, ie (dim 0)."))

(defgeneric (setf rows) (value matrix))

(defgeneric cols (matrix)
  (:documentation "The number of columns, ie (dim 1)."))

(defgeneric (setf cols) (value matrix))


;;; Integral routines for access to matrix store

(declaim (inline matrix-store))

(defgeneric matrix-store (x)
  (:documentation "Returns the store of the matrix"))


