;;; Lisplab, level1-zge.lisp
;;; General, complex double-float matrices

;;; Copyright (C) 2009 Joern Inge Vestgaarden
;;;
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2 of the License, or
;;; (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License along
;;; with this program; if not, write to the Free Software Foundation, Inc.,
;;; 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

(in-package :lisplab)

;;; Complex double float general matrices

(defclass matrix-base-zge 
    (matrix-structure-general matrix-element-complex-double-float matrix-implementation-base) 
   ((matrix-store
    :initarg :store
    :initform nil
    :accessor matrix-store
    :type type-blas-store)))

(defmethod initialize-instance :after ((m matrix-base-zge) &key dim (value 0))
  (with-slots (rows cols size matrix-store) m
    (setf rows (car dim)
	  cols (cadr dim)
	  size (* rows cols))
    (unless matrix-store
      (setf matrix-store (allocate-complex-store size value)))))

(defclass matrix-lisp-zge (matrix-implementation-lisp matrix-base-zge) ()
  (:documentation "A full matrix (rows x cols) with complex double float elements. 
Executes in lisp only."))

(defclass matrix-blas-zge (matrix-implementation-blas matrix-lisp-zge) ()
  (:documentation "A full matrix (rows x cols) with complex double float elements. 
Executes in alien blas/lapack only."))

(defclass matrix-zge (matrix-blas-zge) ()
   (:documentation "A full matrix (rows x cols) with complex double float matrix elements.
Executes first in alien blas/lapack if possible. If not it executes in lisp."))


;;; Add classes to the generic matrix creation scheme
(add-matrix-class 'matrix-base-zge :z :ge :base)
(add-matrix-class 'matrix-lisp-zge :z :ge :lisp)
(add-matrix-class 'matrix-blas-zge :z :ge :blas)
(add-matrix-class 'matrix-zge :z :ge :any)


;;; Level1 methods specialized for zge

(defmethod mref ((matrix matrix-base-zge) row col)
  (ref-blas-complex-store (slot-value matrix 'matrix-store)
			  row col (slot-value matrix 'rows)))

(defmethod (setf mref) (value (matrix  matrix-base-zge) row col)
  (let ((val2 (coerce value '(complex double-float))))
    (declare (type (complex double-float) val2))
    (setf (ref-blas-complex-store (slot-value matrix 'matrix-store) 
				  row col (slot-value matrix 'rows))
	  val2)
    val2))

(defmethod vref ((matrix  matrix-base-zge) i)
  (ref-blas-complex-store (slot-value matrix 'matrix-store) i 0 1))

(defmethod (setf vref) (value (matrix  matrix-base-zge) i)
  (let ((val2 (coerce value '(complex double-float))))
    (declare (type (complex double-float) val2))
    (setf (ref-blas-complex-store (slot-value matrix 'matrix-store) i 0 1)
	  val2)
    val2))

