;;; Lisplab, level2-generic.lisp
;;; Level2, non-specialized methods for functions.

;;; Copyright (C) 2009 Joern Inge Vestgaarden
;;;
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2 of the License, or
;;; (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License along
;;; with this program; if not, write to the Free Software Foundation, Inc.,
;;; 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

(in-package :lisplab)

(defmacro def-each-element-function (name)
  (let ((a (gensym))
	(spec (gensym)))
    `(defmethod ,name ((,a matrix-base))
       (let ((,spec (function-output-type-spec ',name (type-spec ,a))))
	 (mmap ,spec #',name ,a)))))

(define-constant +ordinary-functions-number-to-number-list+ 
    '(.sin .cos .tan 
      .asin .acos .atan 
      .sinh .cosh .tanh
      .asinh .acosh .atanh 
      .re .im .abs .sgn
      .exp .ln .sqr .sqrt .conj ))

(defmacro expand-each-element-ordinary-functions ()
  (cons 'progn
      (mapcar (lambda (name)
		`(def-each-element-function ,name))
	      +ordinary-functions-number-to-number-list+ )))

(expand-each-element-ordinary-functions)

      
;;; Some special functions. Should maybe be separated out.

(defmethod .erf ((a matrix-base))
  (mmap (function-output-type-spec '.erf (type-spec a)) 
	#'.erf
	a))

(defmethod .erfc ((a matrix-base))
  (mmap (function-output-type-spec '.erfc (type-spec a)) 
	#'.erfc
	a))

(defmethod .gamma ((a matrix-base))
  (mmap (function-output-type-spec '.gamma (type-spec a)) 
	#'.gamma
	a))

(defmethod .besj (n (a matrix-base))
  (mmap (function-output-type-spec '.besj (type-spec a)) 
	#'(lambda (x) (.besj n x))
	a))

(defmethod .besj (n (a matrix-base))
  (mmap (function-output-type-spec '.besj (type-spec a)) 
	#'(lambda (x) (.besj n x))
	a))

(defmethod .besj (n (a matrix-base))
  (mmap (function-output-type-spec '.besj (type-spec a)) 
	#'(lambda (x) (.besj n x))
	a))

(defmethod .besy (n (a matrix-base))
  (mmap (function-output-type-spec '.besy (type-spec a)) 
	#'(lambda (x) (.besy n x))
	a))

(defmethod .besi (n (a matrix-base))
  (mmap (function-output-type-spec '.besi (type-spec a)) 
	#'(lambda (x) (.besi n x))
	a))

(defmethod .besk (n (a matrix-base))
  (mmap (function-output-type-spec '.besk (type-spec a)) 
	#'(lambda (x) (.besk n x))
	a))

(defmethod .besh1 (n (a matrix-base))
  (mmap (function-output-type-spec '.besh1 (type-spec a)) 
	#'(lambda (x) (.besh1 n x))
	a))

(defmethod .besh2 (n (a matrix-base))
  (mmap (function-output-type-spec '.besh2 (type-spec a)) 
	#'(lambda (x) (.besh2 n x))
	a))


