;;; Lisplab, level2-interface.lisp
;;; Level2 intefaces.

;;; Copyright (C) 2009 Joern Inge Vestgaarden
;;;
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2 of the License, or
;;; (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License along
;;; with this program; if not, write to the Free Software Foundation, Inc.,
;;; 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

;;; TODO sort and possibly move to other levels

(in-package :lisplab) 

(defgeneric export-list (m)
  (:documentation "Exports the elements of the matrix to a list."))

(defgeneric import-list (m list)
  (:documentation "Imports the elements of the matrix from a list."))

(defgeneric .some (pred a &rest matrices)
  (:documentation "Generalizes some"))

(defgeneric .every (pred a &rest matrices)
  (:documentation "Generalizes every."))

(defgeneric sub-matrix (m rr cc)
  (:documentation "Copies a sub matrix of m. The format of rr = (start stop) or rr = (start step stop)
and the same for the columns."))

(defgeneric copy-contents (a b &optional converter)
  (:documentation "Copies all elements from a to b."))

(defgeneric mnew (class value rows &optional cols)
  (:documentation "General matrix constructor. Creates a new matrix filled with numeric arguments."))

(defgeneric mcreate (a &optional value dim)
  (:documentation "Creates a new matrix of the same type and with the same value as the other,
but with all elements set to value."))

(defgeneric diag (v)
  (:documentation "Creates a diagnoal matrix from the vector."))

(defgeneric to-vector! (a)
  (:documentation "Reshape the object to 1D. Destructive"))

(defgeneric to-vector (a)
  (:documentation "Reshape the object to 1D"))

(defgeneric to-matrix! (a rows)
  (:documentation "Reshape the object to 2D. Destructive"))

(defgeneric to-matrix (a rows)
  (:documentation "Reshape the object to 2D"))

(defgeneric reshape! (a shape)
  (:documentation "Reshapes the object. Detructive"))

(defgeneric reshape (a shape)
  (:documentation "Reshapes the object"))

(defgeneric get-row! (matrix row)
  (:documentation "Gets rows. Destructive"))

(defgeneric get-row (matrix row)
  (:documentation "Gets rows. Destructive"))

(defgeneric get-col! (matrix col)
  (:documentation "Gets rows. Destructive"))

(defgeneric get-col (matrix col)
  (:documentation "Gets rows. Destructive"))

;;; Row operations 

(defgeneric row-swap! (matrix i j)
  (:documentation "Swaps row i and j of matrix. Destructive."))

(defgeneric row-mul! (matrix i number)
  (:documentation "Multiplies row i with number. Destructive."))

(defgeneric row-add! (matrix i j number)
  (:documentation "Adds a multiplicum of row j to row i. A_ic=A_ic+number*A_jc. Destructive.")) 


;;;; Views

(defgeneric view-row (matrix row)
  (:documentation "Returns a shared structure view of the row"))

(defgeneric view-col (matrix col)
  (:documentation "Returns a shared structure view of the row"))

(defgeneric view-matrix (matrix dim &optional (type))
  (:documentation "Returns a shared structure view of the matrix"))

(defgeneric view-transpose (a)
  (:documentation "Returns a transposed matrix with same (shared) elements"))


;;;; Single-element operations

(defgeneric mmap (type f m &rest args)
  (:documentation "Generalization of map, where type = t gives output 
type equals type of m."))

(defgeneric mmap-into (result f m &rest args)
  (:documentation "Generalization of map-into."))

(defgeneric mfill (a value)
  (:documentation "Sets each element to the value. Destructive"))

(defgeneric mmap-operator (f a b output)
  (:documentation "Calls the operator, e.g., .add, .mul, on the elements of the matrices."))

;;; Helpers

(defgeneric msum (m)
  (:documentation "Sums all matrix elements"))

(defgeneric mmin (m)
  (:documentation "Retuns the smalles matrix element"))

(defgeneric mmax (m)
  (:documentation "Retuns the largest matrix element"))

(defgeneric mabsmin (m)
  (:documentation "Retuns the matrix element closest to zero"))

(defgeneric mabsmax (m)
  (:documentation "Retuns the matrix element with largest absolute value"))

(defgeneric mminmax (m)
  (:documentation "Retuns a list with (minumum maximum)"))

(defgeneric circ-shift (m shifts)
  (:documentation "Shifts the matrix with periodic indecices"))

(defgeneric pad-shift (m shifts &optional value)
  (:documentation "Shifts the matrix and pads results"))

(defgeneric mreverse (m)
  (:documentation "Reverts elements of matrix or vector. Similar to cl:reverse"))