;;; Lisplab, level2-operator.lisp
;;; Level2, non-specialized methods.

;;; Copyright (C) 2009 Joern Inge Vestgaarden
;;;
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2 of the License, or
;;; (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License along
;;; with this program; if not, write to the Free Software Foundation, Inc.,
;;; 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

(in-package :lisplab)

(defmethod .complex ((a matrix-base) (b matrix-base))
  (.+ a (.* %i b)))
(defmethod .complex ((a matrix-base) b)
  (.+ a (.* %i b)))
(defmethod .complex (a (b matrix-base))
  (.+ a (.* %i b)))

;;;; Basic boolean operators

(defmethod .= ((a matrix-base) (b matrix-base) &optional acc)
  (if acc
      (.every (lambda (a b) (.= a b acc)) a b)
      (.every #'.= a b)))

(defmethod .= ((a matrix-base) (b number) &optional acc)
  (if acc
      (.every (lambda (a) (.= a b acc)) a)
      (.every (lambda (a) (.= a b)) a)))

(defmethod .= ((a number) (b matrix-base) &optional acc)
  (if acc
      (.every (lambda (b) (.= a b acc)) b)
      (.every (lambda (b) (.= a b)) b)))

(defmethod ./= ((a matrix-base) (b matrix-base) &optional acc)
  (not (.= a b acc)))

(defmethod ./= ((a matrix-base) (b number) &optional acc)
  (not (.= a b acc)))

(defmethod ./= ((a number) (b matrix-base) &optional acc)
  (not (.= a b acc)))

(defmacro def-matrix-base-boolean-operator (op)
  (let ((a (gensym))
	(b (gensym)))
    `(progn
       (defmethod ,op ((,a matrix-base) (,b matrix-base))
	 (.every #',op ,a ,b))
       (defmethod ,op ((,a matrix-base) (,b number))
	 (.every (lambda (,a) (,op ,a ,b)) ,a))
       (defmethod ,op ((,a number) (,b matrix-base))	 
	 (.every (lambda (,b) (,op ,a ,b)) ,b)))))

(def-matrix-base-boolean-operator .<)

(def-matrix-base-boolean-operator .<=)

(def-matrix-base-boolean-operator .>)

(def-matrix-base-boolean-operator .>=)

;;; Element-wice operators

(defmethod mmap-operator (op (a matrix-base) b output)
  (mmap-into output (lambda (x) (funcall op x b)) a))

(defmethod mmap-operator (op a (b matrix-base) output)
  (mmap-into output (lambda (x) (funcall op a x)) b))

(defmethod mmap-operator (op (a matrix-base) (b matrix-base) output)
  (mmap-into output op a b))

(defmacro defmethod-operator-matrix-matrix (name)
  (let ((a (gensym))
	(b (gensym))
	(out (gensym))
	(spec (gensym)))
    `(defmethod ,name ((,a matrix-base) (,b matrix-base))
       (let* ((,spec (operator-output-type-spec ',name 
					       (type-spec ,a) 
					       (type-spec ,b)))
	      (,out (make-matrix-instance ,spec (dim ,a) 0)))
	 (mmap-operator #',name ,a ,b ,out))))) 

(defmacro defmethod-operator-matrix-any (name)
  (let ((a (gensym))
	(b (gensym))
	(out (gensym))
	(spec (gensym)))
    `(defmethod ,name ((,a matrix-base) ,b)
       (let* ((,spec (operator-output-type-spec ',name 
					       (type-spec ,a) 
					       (type-spec ,b)))
	      (,out (make-matrix-instance ,spec (dim ,a) 0)))
	 (mmap-operator #',name ,a ,b ,out))))) 

(defmacro defmethod-operator-any-matrix (name)
  (let ((a (gensym))
	(b (gensym))
	(out (gensym))
	(spec (gensym)))
    `(defmethod ,name (,a (,b matrix-base))
       (let* ((,spec (operator-output-type-spec ',name 
					       (type-spec ,a) 
					       (type-spec ,b)))
	      (,out (make-matrix-instance ,spec (dim ,b) 0)))
	 (mmap-operator #',name ,a ,b ,out))))) 

(defmacro def-each-element-operator (name)
  "Makes so that the binary operator can map element-wice."
  `(progn
     (defmethod-operator-matrix-matrix ,name)
     (defmethod-operator-matrix-any ,name)
     (defmethod-operator-any-matrix ,name)
     'thats-it))
     
(def-each-element-operator .add)
(def-each-element-operator .mul)
(def-each-element-operator .div)
(def-each-element-operator .sub)
(def-each-element-operator .expt)
(def-each-element-operator .max)
(def-each-element-operator .min)

